/*****************************************************************************
 * Copyright  2018 NXP
 * All rights reserved
 *

 * SPDX-License-Identifier: LA_OPT_BASE_LICENSE v24 May 2018
 *****************************************************************************/
/*****************************************************************************
 *
 * MODULE:             JN-AN-1245
 *
 * COMPONENT:          app_buttons.h
 *
 * DESCRIPTION:        ZLO Demo: DK button handler -Interface
 *
 ****************************************************************************
 *
 * This software is owned by NXP B.V. and/or its supplier and is protected
 * under applicable copyright laws. All rights are reserved. We grant You,
 * and any third parties, a license to use this software solely and
 * exclusively on NXP products [NXP Microcontrollers such as JN5168, JN5169,
 * JN5179, JN5189].
 * You, and any third parties must reproduce the copyright and warranty notice
 * and any other legend of ownership on each copy or partial copy of the
 * software.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright NXP B.V. 2016-2018. All rights reserved
 *
 ***************************************************************************/

#ifndef APP_BUTTONS_H
#define APP_BUTTONS_H

/****************************************************************************/
/***        Macro Definitions                                             ***/
/****************************************************************************/
/* Number of real GPIO lines */
#define APP_NUMBER_OF_GPIO 22
/* Dummy GPIO bit to use for internal NTAG FD line */
/* Use 22 as it has special purpose of waking from sleep when masks below are provided to PWRM */
#define APP_INTERNAL_NTAG_FD_BIT (22)
/* PIO pin to use for FD line */
/* 0xff = disabled, <=21 = GPIO, 22 = JN5189T Internal NTAG FD */
#define APP_BUTTONS_NFC_FD (APP_INTERNAL_NTAG_FD_BIT)

/****************************************************************************/
/***        Type Definitions                                              ***/
/****************************************************************************/

#if (defined BUTTON_MAP_OM15082)
    #if ((defined APP_NTAG_ICODE) && (APP_BUTTONS_NFC_FD != (0xff)))
        typedef enum {
            APP_E_BUTTONS_BUTTON_1 = 0,
            APP_E_BUTTONS_BUTTON_SW1,
            APP_E_BUTTONS_BUTTON_SW2,
            APP_E_BUTTONS_BUTTON_SW3,
            APP_E_BUTTONS_BUTTON_SW4,
            APP_E_BUTTONS_NFC_FD
        } APP_teButtons;

        #define APP_BUTTONS_NUM                     (6UL)

        #define APP_BUTTONS_DIO_MASK                ((1 << APP_BOARD_SW0_PIN)|(1 << APP_BOARD_SW1_PIN)|(1 << APP_BOARD_SW2_PIN) | (1 << APP_BOARD_SW3_PIN) | (1 << APP_BOARD_SW4_PIN) | (1 << APP_BUTTONS_NFC_FD))
        #define APP_BUTTONS_DIO_MASK_FOR_DEEP_SLEEP (                         (1 << APP_BOARD_SW1_PIN)|(1 << APP_BOARD_SW2_PIN) | (1 << APP_BOARD_SW3_PIN) | (1 << APP_BOARD_SW4_PIN) | (1 << APP_BUTTONS_NFC_FD))
        /* OM15076-3 Carrier Board */
        #define APP_BUTTON_INVERT_MASK                0
        /* OM15076-1 Carrier Board */
        //#define APP_BUTTON_INVERT_MASK              (1 << APP_BOARD_SW0_PIN)
        #define DEMO_GINT0_POL_MASK         ((APP_BUTTON_INVERT_MASK) & 0x3FFFFF)
        #define DEMO_GINT0_ENA_MASK         ((APP_BUTTONS_DIO_MASK)   & 0x3FFFFF)
    #else
        typedef enum {
            APP_E_BUTTONS_BUTTON_1 = 0,
            APP_E_BUTTONS_BUTTON_SW1,
            APP_E_BUTTONS_BUTTON_SW2,
            APP_E_BUTTONS_BUTTON_SW3,
            APP_E_BUTTONS_BUTTON_SW4
        } APP_teButtons;

        #define APP_BUTTONS_NUM                     (5UL)

        #define APP_BUTTONS_DIO_MASK                ((1 << APP_BOARD_SW0_PIN)|(1 << APP_BOARD_SW1_PIN)|(1 << APP_BOARD_SW2_PIN) | (1 << APP_BOARD_SW3_PIN) | (1 << APP_BOARD_SW4_PIN))
        #define APP_BUTTONS_DIO_MASK_FOR_DEEP_SLEEP (                         (1 << APP_BOARD_SW1_PIN)|(1 << APP_BOARD_SW2_PIN) | (1 << APP_BOARD_SW3_PIN) | (1 << APP_BOARD_SW4_PIN))
        /* OM15076-3 Carrier Board */
        #define APP_BUTTON_INVERT_MASK                0
        /* OM15076-1 Carrier Board */
        //#define APP_BUTTON_INVERT_MASK              (1 << APP_BOARD_SW0_PIN)
        #define DEMO_GINT0_POL_MASK         ((APP_BUTTON_INVERT_MASK) & 0x3FFFFF)
        #define DEMO_GINT0_ENA_MASK         ((APP_BUTTONS_DIO_MASK)   & 0x3FFFFF)
    #endif
#elif (defined BUTTON_MAP_K32W148EVK)
     typedef enum {
            APP_E_BUTTONS_BUTTON_0 = 0,
	     APP_E_BUTTONS_BUTTON_1,
	     APP_E_BUTTONS_BUTTON_2,
	     APP_E_BUTTONS_BUTTON_3,
	     APP_E_BUTTONS_BUTTON_4,
        } APP_teButtons;

    #define APP_BUTTONS_NUM             (5UL)
	
    #define APP_BUTTONS_BUTTON_0_GPIO GPIOC
    #define APP_BUTTONS_BUTTON_1_GPIO GPIOC
    #define APP_BUTTONS_BUTTON_2_GPIO GPIOC
    #define APP_BUTTONS_BUTTON_3_GPIO GPIOA
    #define APP_BUTTONS_BUTTON_4_GPIO GPIOB
	
    #define APP_BUTTONS_BUTTON_0_PORT PORTC
    #define APP_BUTTONS_BUTTON_1_PORT PORTC
    #define APP_BUTTONS_BUTTON_2_PORT PORTC
    #define APP_BUTTONS_BUTTON_3_PORT PORTA
    #define APP_BUTTONS_BUTTON_4_PORT PORTB
	
    #define APP_BUTTONS_BUTTON_0_PIN    (6UL)
    #define APP_BUTTONS_BUTTON_1_PIN    (1UL)
    #define APP_BUTTONS_BUTTON_2_PIN    (4UL)
    #define APP_BUTTONS_BUTTON_3_PIN    (21UL)
    #define APP_BUTTONS_BUTTON_4_PIN    (4UL)

    #define APP_BUTTONS_BUTTON_0_IRQ    GPIOC_INT0_IRQn
    #define APP_BUTTONS_BUTTON_1_IRQ    GPIOC_INT0_IRQn
    #define APP_BUTTONS_BUTTON_2_IRQ    GPIOC_INT0_IRQn
    #define APP_BUTTONS_BUTTON_3_IRQ    GPIOA_INT0_IRQn
    #define APP_BUTTONS_BUTTON_4_IRQ    GPIOB_INT0_IRQn
	
    #define APP_BUTTONS_DIO_MASK    (0x1F)
	
    #define APP_BUTTONS_BUTTON_0_WAKE_INDEX   (11U)
    #define APP_BUTTONS_BUTTON_1_WAKE_INDEX   (8U)
    #define APP_BUTTONS_BUTTON_2_WAKE_INDEX   (10U)
    #define APP_BUTTONS_BUTTON_3_WAKE_INDEX   (5U)
    #define APP_BUTTONS_BUTTON_4_WAKE_INDEX   (15U)

    #define BUTTON_ON_GPIOA_IRQ_HANDLER  GPIOA_INT0_IRQHandler
    #define BUTTON_ON_GPIOB_IRQ_HANDLER  GPIOB_INT0_IRQHandler
    #define BUTTON_ON_GPIOC_IRQ_HANDLER  GPIOC_INT0_IRQHandler
    #define BUTTON_ON_GPIOD_IRQ_HANDLER  GPIOD_INT0_IRQHandler
#else
    typedef enum {
        APP_E_BUTTONS_BUTTON_1
    } APP_teButtons;

    #define APP_BUTTONS_NUM                     (1UL)
    #define APP_BUTTONS_BUTTON_1            (4)

    #define APP_BUTTONS_DIO_MASK                (1 << APP_BUTTONS_BUTTON_1)
#endif

typedef enum {
    E_INTERRUPT_UNKNOWN,
    E_INTERRUPT_BUTTON,
    E_INTERRUPT_WAKE_TIMER_EXPIRY
} teInterruptType;

/****************************************************************************/
/***        Exported Functions                                            ***/
/****************************************************************************/
PUBLIC bool_t APP_bButtonInitialise(void);
extern PUBLIC void vManageWakeUponSysControlISR(teInterruptType eInterruptType);
PUBLIC uint32 APP_u32GetSwitchIOState(void);
PUBLIC void APP_cbTimerButtonScan(void *pvParam);
PUBLIC void APP_cbTimerButtonDelay(void *pvParam);

/****************************************************************************/
/***        External Variables                                            ***/
/****************************************************************************/

/****************************************************************************/
/****************************************************************************/
/****************************************************************************/

#endif /*APP_BUTTONS_H*/
