/*****************************************************************************
 * Copyright  2018 NXP
 * All rights reserved
 *

 * SPDX-License-Identifier: LA_OPT_BASE_LICENSE v24 May 2018
 *****************************************************************************/
/****************************************************************************
 *
 * MODULE:             UART
 *
 * COMPONENT:          Uart interface for BDB Application Template AN1217 (Coorinator)
 *
 * VERSION:
 *
 * REVISION:
 *
 * DATED:
 *
 * STATUS:
 *
 * AUTHOR:
 *
 * DESCRIPTION:        Hardware abstarction layer for UART peripheral
 *
 *
 ****************************************************************************
 *
 * This software is owned by NXP B.V. and/or its supplier and is protected
 * under applicable copyright laws. All rights are reserved. We grant You,
 * and any third parties, a license to use this software solely and
 * exclusively on NXP products [NXP Microcontrollers such as JN5168, JN5169,
 * JN5179, JN5189].
 * You, and any third parties must reproduce the copyright and warranty notice
 * and any other legend of ownership on each copy or partial copy of the
 * software.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright NXP B.V. 2016-2018. All rights reserved
 *
 ****************************************************************************/

/****************************************************************************/
/***        Include files                                                 ***/
/****************************************************************************/

#include <jendefs.h>
#include "board.h"
#include "app.h"
#ifndef K32W148EVK
#include "fsl_usart.h"
#endif
#include "app_uart.h"
#include "ZQueue.h"
#include "portmacro.h"
#include "app_main.h"

#ifdef K32W148EVK
#include "pin_mux.h"
#include "clock_config.h"
#include "board.h"
#include "fsl_lpuart_edma.h"
#if defined(FSL_FEATURE_SOC_DMAMUX_COUNT) && FSL_FEATURE_SOC_DMAMUX_COUNT
#include "fsl_dmamux.h"
#endif
/*******************************************************************************
 * Definitions
 ******************************************************************************/
#define LPUART_TX_DMA_CHANNEL       0U
#define LPUART_RX_DMA_CHANNEL       1U
#define LPUART_TX_EDMA_CHANNEL     32U
#define LPUART_RX_EDMA_CHANNEL     31U
#endif

/****************************************************************************/
/***        Macro Definitions                                             ***/
/****************************************************************************/

#ifndef TRACE_UART
#define TRACE_UART  FALSE
#endif

/****************************************************************************/
/***        Type Definitions                                              ***/
/****************************************************************************/

/****************************************************************************/
/***        Local Function Prototypes                                     ***/
/****************************************************************************/

/****************************************************************************/
/***        Exported Variables                                            ***/
/****************************************************************************/

/****************************************************************************/
/***        Local Variables                                               ***/
/****************************************************************************/

/****************************************************************************/
/***        Exported Functions                                            ***/
/****************************************************************************/

/****************************************************************************/
/***        Local Functions                                               ***/
/****************************************************************************/
#ifdef K32W148EVK
/****************************************************************************/
/***        Type Definitions                                              ***/
/****************************************************************************/

/****************************************************************************/
/***        Local Function Prototypes                                     ***/
/****************************************************************************/
PRIVATE void LPUART_DMA_Init();
/****************************************************************************/
/***        Exported Variables                                            ***/
/****************************************************************************/

/****************************************************************************/
/***        Local Variables                                               ***/
/****************************************************************************/

/****************************************************************************/
/***        Exported Functions                                            ***/
/****************************************************************************/

/****************************************************************************/
/***        Local Functions                                               ***/
/****************************************************************************/
/*******************************************************************************
 * Prototypes
 ******************************************************************************/
/* LPUART user callback */
PRIVATE void LPUART_UserCallback(LPUART_Type *base, lpuart_edma_handle_t *handle, status_t status, void *userData);

/*******************************************************************************
 * Variables
 ******************************************************************************/

lpuart_edma_handle_t g_lpuartEdmaHandle;
edma_handle_t g_lpuartTxEdmaHandle;
edma_handle_t g_lpuartRxEdmaHandle;
AT_NONCACHEABLE_SECTION_INIT(uint8_t g_tipString[]) =
    "LPUART EDMA example\r\nSend back received data\r\nEcho every 8 characters\r\n";
AT_NONCACHEABLE_SECTION_INIT(uint8_t g_txBuffer[1]) = {0};
AT_NONCACHEABLE_SECTION_INIT(uint8_t g_rxBuffer[1]) = {0};
volatile bool txOnGoing                                              = false;
volatile bool rxOnGoing                                              = false;
#endif

/****************************************************************************
 *
 * NAME: vUART_Init
 *
 * DESCRIPTION:
 *
 * PARAMETERS:      Name            RW  Usage
 * None.
 *
 * RETURNS:
 * None.
 *
 * NOTES:
 * None.
 ****************************************************************************/
PUBLIC void UART_vInit(void)
{
#ifndef K32W148EVK
    usart_config_t config;
    DBG_vPrintf(TRACE_UART, "\r\nInitialising UART ...");

    USART_GetDefaultConfig(&config);
    config.baudRate_Bps = UART_BAUD_RATE;
    config.enableTx = true;
    config.enableRx = true;

    USART_Init(UART, &config, CLOCK_GetFreq(kCLOCK_Fro32M));

    /* Enable RX interrupt. */
    USART_EnableInterrupts(UART, kUSART_RxLevelInterruptEnable | kUSART_RxErrorInterruptEnable);
    EnableIRQ(UART1_IRQ);
    DBG_vPrintf(TRACE_UART, "\r\nDone");
#else
    LPUART_DMA_Init();
#endif
}
#ifdef K32W148EVK
/* LPUART user callback */
PRIVATE void LPUART_UserCallback(LPUART_Type *base, lpuart_edma_handle_t *handle, status_t status, void *userData)
{
    userData = userData;

    if (kStatus_LPUART_TxIdle == status)
    {
        txOnGoing    = false;
	 uint8 u8Byte;
	 if (TRUE == ZQ_bQueueReceive(&APP_msgSerialTx, &u8Byte)) {
            UART_vTxChar(u8Byte);
            DBG_vPrintf(TRACE_UART, "\r\ntx u8Byte %x", u8Byte);
        }
        else
        {
            /* disable tx interrupt as nothing to send */
            UART_vSetTxInterrupt(FALSE);
        }
    }

    if (kStatus_LPUART_RxIdle == status)
    {
        rxOnGoing     = false;
    }
}
PRIVATE void LPUART_DMA_Init()
{
    lpuart_config_t lpuartConfig;
    edma_config_t config;
    CLOCK_SetIpSrc(kCLOCK_Lpuart0, kCLOCK_IpSrcFro6M);
    /* Initialize the LPUART. */
    /*
     * lpuartConfig.baudRate_Bps = 115200U;
     * lpuartConfig.parityMode = kLPUART_ParityDisabled;
     * lpuartConfig.stopBitCount = kLPUART_OneStopBit;
     * lpuartConfig.txFifoWatermark = 0;
     * lpuartConfig.rxFifoWatermark = 0;
     * lpuartConfig.enableTx = false;
     * lpuartConfig.enableRx = false;
     */
     
    LPUART_GetDefaultConfig(&lpuartConfig);
    lpuartConfig.baudRate_Bps = BOARD_DEBUG_UART_BAUDRATE;
    lpuartConfig.enableTx     = true;
    lpuartConfig.enableRx     = true;

    LPUART_Init(LPUART0, &lpuartConfig, BOARD_DEBUG_UART_CLK_FREQ);

    /* Init the EDMA module */
    EDMA_GetDefaultConfig(&config);
    EDMA_Init(DMA0, &config);
    EDMA_CreateHandle(&g_lpuartTxEdmaHandle, DMA0, LPUART_TX_DMA_CHANNEL);
    EDMA_CreateHandle(&g_lpuartRxEdmaHandle, DMA0, LPUART_RX_DMA_CHANNEL);
#if defined(FSL_FEATURE_EDMA_HAS_CHANNEL_MUX) && FSL_FEATURE_EDMA_HAS_CHANNEL_MUX
    EDMA_SetChannelMux(DMA0, LPUART_TX_DMA_CHANNEL, LPUART_TX_EDMA_CHANNEL);
    EDMA_SetChannelMux(DMA0, LPUART_RX_DMA_CHANNEL, LPUART_RX_EDMA_CHANNEL);
#endif
    /* Create LPUART DMA handle. */
    LPUART_TransferCreateHandleEDMA(LPUART0, &g_lpuartEdmaHandle, LPUART_UserCallback, NULL, &g_lpuartTxEdmaHandle,
                                    &g_lpuartRxEdmaHandle);

}
#endif
#ifndef K32W148EVK
/****************************************************************************
 *
 * NAME: vUART_SetBuadRate
 *
 * DESCRIPTION:
 *
 * PARAMETERS: Name        RW  Usage
 *
 * RETURNS:
 *
 ****************************************************************************/

PUBLIC void UART_vSetBaudRate(uint32 u32BaudRate)
{
    int result;

    /* setup baudrate */
    result = USART_SetBaudRate(UART, u32BaudRate, CLOCK_GetFreq(kCLOCK_Fro32M));
    if (kStatus_Success != result)
    {
        //DBG_vPrintf(TRACE_UART,"\r\nFailed to set UART speed ");
    }
}
/****************************************************************************
 *
 * NAME: UartIsr
 *
 * DESCRIPTION: Handle interrupts from uart
 *
 * PARAMETERS:      Name            RW  Usage
 * None.
 *
 * RETURNS:
 * None.
 *
 * NOTES:
 * None.
 ****************************************************************************/
 void APP_isrUart ( void )
{
    uint8 u8Byte;
    if ((kUSART_RxFifoNotEmptyFlag | kUSART_RxError) & USART_GetStatusFlags(UART))
    {
        uint8 u8Byte = USART_ReadByte(UART);
        ZQ_bQueueSend(&APP_msgSerialRx, &u8Byte);
    }
    else if ((kUSART_TxFifoNotFullFlag & USART_GetStatusFlags(UART)))
    {
        if (TRUE == ZQ_bQueueReceive(&APP_msgSerialTx, &u8Byte)) {
            USART_WriteByte(UART, u8Byte);
            DBG_vPrintf(TRACE_UART, "\r\ntx u8Byte %x", u8Byte);

        }
        else
        {
            /* disable tx interrupt as nothing to send */
            UART_vSetTxInterrupt(FALSE);
        }
    }
}
#endif
/****************************************************************************
 *
 * NAME: UART_vRtsStopFlow
 *
 * DESCRIPTION:
 * Set UART RS-232 RTS line high to stop any further data coming in
 *
 ****************************************************************************/

PUBLIC void UART_vRtsStopFlow(void)
{
    //vAHI_UartSetControl(UART, FALSE, FALSE, E_AHI_UART_WORD_LEN_8, TRUE, E_AHI_UART_RTS_HIGH);
}

/****************************************************************************
 *
 * NAME: UART_vRtsStartFlow
 *
 * DESCRIPTION:
 * Set UART RS-232 RTS line low to allow further data
 *
 ****************************************************************************/

PUBLIC void UART_vRtsStartFlow(void)
{
    //vAHI_UartSetControl(UART, FALSE, FALSE, E_AHI_UART_WORD_LEN_8, TRUE, E_AHI_UART_RTS_LOW);
}
/* [I SP001222_P1 283] end */

/****************************************************************************
 *
 * NAME: vUART_TxChar
 *
 * DESCRIPTION:
 * Set UART RS-232 RTS line low to allow further data
 *
 ****************************************************************************/
PUBLIC void UART_vTxChar(uint8 u8Char)
{
#ifndef K32W148EVK
    USART_WriteByte(UART, u8Char);
    /* Wait to finish transfer */
    while (!(UART->FIFOSTAT & USART_FIFOSTAT_TXEMPTY_MASK))
    {
    }
#else
    lpuart_transfer_t sendXfer;
    memcpy(g_txBuffer,&u8Char,1);
    sendXfer.data        = g_txBuffer;
    sendXfer.dataSize    = 1;
    txOnGoing = true;
    LPUART_SendEDMA(LPUART0, &g_lpuartEdmaHandle, &sendXfer);
#endif
}

/****************************************************************************
 *
 * NAME: vUART_TxReady
 *
 * DESCRIPTION:
 * Set UART RS-232 RTS line low to allow further data
 *
 ****************************************************************************/
PUBLIC bool_t UART_bTxReady()
{
#ifndef K32W148EVK
    return ( (kUSART_TxFifoEmptyFlag|kUSART_TxFifoNotFullFlag) & USART_GetStatusFlags(UART) );
#else
    return (!txOnGoing);
#endif
}

/****************************************************************************
 *
 * NAME: vUART_SetTxInterrupt
 *
 * DESCRIPTION:
 * Enable / disable the tx interrupt
 *
 ****************************************************************************/
PUBLIC void UART_vSetTxInterrupt(bool_t bState)
{
#ifndef K32W148EVK
    USART_DisableInterrupts(UART, (kUSART_TxErrorInterruptEnable | kUSART_RxErrorInterruptEnable | kUSART_TxLevelInterruptEnable| kUSART_RxLevelInterruptEnable) );
    if (bState)
    {
        USART_EnableInterrupts(UART, (kUSART_RxLevelInterruptEnable | kUSART_RxErrorInterruptEnable) );
    }
    else
    {
        USART_EnableInterrupts(UART, (kUSART_RxLevelInterruptEnable | kUSART_RxErrorInterruptEnable | kUSART_TxLevelInterruptEnable|kUSART_TxErrorInterruptEnable ));
    }
#endif
}
#ifdef K32W148EVK
/****************************************************************************
 *
 * NAME: UART_bBufferReceive
 *
 * DESCRIPTION:
 *
 * PARAMETERS: Name        RW  Usage
 *
 * RETURNS:
 *
 ****************************************************************************/
PUBLIC bool_t UART_bBufferReceive ( uint8* pu8Data )
{
      uint32    u32Storage;
      ZPS_eEnterCriticalSection ( NULL, &u32Storage);
      status_t status;
      lpuart_transfer_t receiveXfer;
      receiveXfer.data     = g_rxBuffer;
      receiveXfer.dataSize = 1;
      //if (!rxOnGoing) 
      {
            rxOnGoing = true;
            status = LPUART_ReceiveEDMA(LPUART0, &g_lpuartEdmaHandle, &receiveXfer);
	     if(status == kStatus_Success)
	     	memcpy(pu8Data,g_rxBuffer,1);
      }
      ZPS_eExitCriticalSection ( NULL, &u32Storage);

	return (status == kStatus_Success) ? TRUE:FALSE;
}
#endif
/****************************************************************************/
/***        END OF FILE                                                   ***/
/****************************************************************************/
