/*****************************************************************************
 * Copyright  2018 NXP
 * All rights reserved
 *

 * SPDX-License-Identifier: LA_OPT_BASE_LICENSE v24 May 2018
 *****************************************************************************/
/*****************************************************************************
 *
 * MODULE:             JN-AN-1245
 *
 * COMPONENT:          app_zcl_remote_task.c
 *
 * DESCRIPTION:        ZLO Demo: ZigBee Cluster Handlers - Implementation
 *
 ****************************************************************************
 *
 * This software is owned by NXP B.V. and/or its supplier and is protected
 * under applicable copyright laws. All rights are reserved. We grant You,
 * and any third parties, a license to use this software solely and
 * exclusively on NXP products [NXP Microcontrollers such as JN5168, JN5169,
 * JN5179, JN5189].
 * You, and any third parties must reproduce the copyright and warranty notice
 * and any other legend of ownership on each copy or partial copy of the
 * software.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright NXP B.V. 2016-2018. All rights reserved
 *
 ***************************************************************************/

/****************************************************************************/
/***        Include files                                                 ***/
/****************************************************************************/

#include <jendefs.h>
#include <AppApi.h>
#include "pdum_apl.h"
#include "pdum_gen.h"
#include "PDM.h"
#include "dbg.h"

#include "zps_apl_af.h"
#include "zps_apl_zdo.h"
#include "zps_apl_aib.h"
#include "zps_apl_zdp.h"
#include "rnd_pub.h"
#include "mac_pib.h"
#include "string.h"

#include "zcl_options.h"
#include "zll_commission.h"
#include "Basic.h"
#include "commission_endpoint.h"
#include "app_common.h"
#include "zlo_controller_node.h"
#ifdef CLD_OTA
    #include "app_ota_client.h"
#endif

#include "app_events.h"
#include "app_main.h"
#include "ZQueue.h"
#include "bdb_api.h"
#include "ZTimer.h"

#ifdef DEBUG_ZCL
#define TRACE_ZCL   TRUE
#else
#define TRACE_ZCL   FALSE
#endif

#ifdef DEBUG_REMOTE_TASK
#define TRACE_REMOTE_TASK   TRUE
#else
#define TRACE_REMOTE_TASK   FALSE
#endif

#ifdef DEBUG_LIGHT_AGE
#define TRACE_LIGHT_AGE   TRUE
#else
#define TRACE_LIGHT_AGE   FALSE
#endif

#define TRACE_RESET   FALSE

/****************************************************************************/
/***        Macro Definitions                                             ***/
/****************************************************************************/

/****************************************************************************/
/***        Type Definitions                                              ***/
/****************************************************************************/

#define ZCL_TICK_TIME           ZTIMER_TIME_MSEC(100)
#define RADIO_TEMP_UPDATE_MS 300000

/****************************************************************************/
/***        Local Function Prototypes                                     ***/
/****************************************************************************/

PRIVATE void APP_ZCL_cbGeneralCallback(tsZCL_CallBackEvent *psEvent);
PRIVATE void APP_ZCL_cbEndpointCallback(tsZCL_CallBackEvent *psEvent);
PRIVATE void vStartEffect(uint8 u8Effect);
PRIVATE void APP_ZCL_cbZllUtilityCallback(tsZCL_CallBackEvent *psEvent);

/****************************************************************************/
/***        Exported Variables                                            ***/
/****************************************************************************/

/****************************************************************************/
/***        Local Variables                                               ***/
/****************************************************************************/
PRIVATE uint32 u32RadioTempUpdateMs = 0;

/****************************************************************************/
/***        Exported Functions                                            ***/
/****************************************************************************/

PUBLIC void* psGetDeviceTable(void) {
    return &sDeviceTable;
}

/****************************************************************************
 *
 * NAME: APP_ZCL_vInitialise
 *
 * DESCRIPTION:
 * Initialises ZCL related functions
 *
 * RETURNS:
 * void
 *
 ****************************************************************************/
PUBLIC void APP_ZCL_vInitialise(bool_t bColdRestart)
{
    teZCL_Status eZCL_Status;

    /* Initialise ZLL */
    eZCL_Status = eZCL_Initialise(&APP_ZCL_cbGeneralCallback, apduZCL);
    if (eZCL_Status != E_ZCL_SUCCESS)
    {
        DBG_vPrintf(TRACE_ZCL, "Error: eZCL_Initialise returned %d\r\n", eZCL_Status);
    }

    /* Register Commission EndPoint */
    eZCL_Status = eApp_ZLO_RegisterEndpoint(&APP_ZCL_cbEndpointCallback);
    if (eZCL_Status != E_ZCL_SUCCESS)
    {
            DBG_vPrintf(TRACE_ZCL, "Error: eZLL_RegisterCommissionEndPoint:%d\r\n", eZCL_Status);
    }

    DBG_vPrintf(TRACE_REMOTE_TASK, "Chan Mask %08x\n", ZPS_psAplAibGetAib()->pau32ApsChannelMask[0]);
    DBG_vPrintf(TRACE_REMOTE_TASK, "\nRxIdle TRUE");

#ifdef K32W148EVK
    ZPS_vMacPibSetRxOnWhenIdle(TRUE,FALSE);
#else
    MAC_vPibSetRxOnWhenIdle(NULL, TRUE, FALSE);
#endif

    sDeviceTable.asDeviceRecords[0].u64IEEEAddr = ZPS_u64NwkNibGetExtAddr(ZPS_pvAplZdoGetNwkHandle() );
    DBG_vPrintf(TRACE_REMOTE_TASK, "\ntsCLD_Groups %d", sizeof(tsCLD_Groups));
    DBG_vPrintf(TRACE_REMOTE_TASK, "\ntsCLD_GroupTableEntry %d", sizeof(tsCLD_GroupTableEntry));

    vAPP_ZCL_DeviceSpecific_Init();

    if(bColdRestart)
    {
    #ifdef CLD_OTA
        vAppInitOTA();
    #endif
        /* Start the tick timer */
        ZTIMER_eStart(u8TimerZCL, ZCL_TICK_TIME);
    }
    else
    {
         #if (defined SLEEP_MIN_RETENTION) && (defined CLD_OTA) && (defined OTA_CLIENT)
             vZCL_SetUTCTime(U32UTCTimeBeforeSleep);
             bInitialiseOTAClusterAndAttributes();
            /*ZCL time data is not retained so , OTA is. This make OTA think ti's registered while
             * it isn't.
             */
            extern  void vOtaTimerClickCallback(
                    tsZCL_CallBackEvent         *psCallBackEvent);
            if(eZCL_TimerRegister(E_ZCL_TIMER_CLICK_MS, 0, vOtaTimerClickCallback)!= E_ZCL_SUCCESS)
            {
                DBG_vPrintf(TRACE_REMOTE_TASK, "Failed to register the timer\n");
            }

        #endif
    }
}

/****************************************************************************
 *
 * NAME: APP_ZCL_vInitialise
 *
 * DESCRIPTION:
 * Initialises ZCL related functions
 *
 * RETURNS:
 * void
 *
 ****************************************************************************/
PUBLIC void APP_cbTimerZclTick(void *pvParam)
{
    static uint32 u32Tick1Sec = 0;
    static uint32 u32RadioTempUpdateMs = 0;

    tsZCL_CallBackEvent sCallBackEvent;
    ZTIMER_eStart(u8TimerZCL, ZCL_TICK_TIME);

    u32Tick1Sec++;

    eZCL_Update100mS();

    /* Wrap the 1 second  counter and provide 1Hz ticks to cluster */
    if(u32Tick1Sec == 10)
    {
        u32Tick1Sec = 0;
        sCallBackEvent.pZPSevent = NULL;
        sCallBackEvent.eEventType = E_ZCL_CBET_TIMER;
        vZCL_EventHandler(&sCallBackEvent);

    #ifdef CLD_OTA
        vRunAppOTAStateMachine(1000);
    #endif
    }
	
#ifndef K32W148EVK
    u32RadioTempUpdateMs += 100; /* 100ms */

    /* Need to update radio temp ? */
    if(u32RadioTempUpdateMs >= RADIO_TEMP_UPDATE_MS)
    {
        /* Reset timer counter */
        u32RadioTempUpdateMs = 0;
        /* Update radio temperature (not loading calibration) */
        APP_vRadioTempUpdate(FALSE);
    }
#endif
}
/****************************************************************************
 *
 * NAME: APP_vRadioTempUpdateMs
 *
 * DESCRIPTION:
 * Updates radio temperature timer (for use after sleeping)
 *
 * RETURNS:
 * void
 *
 ****************************************************************************/
 PUBLIC void APP_vRadioTempUpdateMs(uint32 u32Ms)
{
    /* Update timer */
    u32RadioTempUpdateMs += u32Ms;
}

/****************************************************************************
 *
 * NAME: APP_ZCL_vEventHandler
 *
 * DESCRIPTION:
 * Handles events from the ZigBee stack
 *
 * RETURNS:
 * void
 *
 ****************************************************************************/
PUBLIC void APP_ZCL_vEventHandler(ZPS_tsAfEvent *psStackEvent)
{

    //tsBDB_ZCLEvent sCommissionEvent;
    tsZCL_CallBackEvent sCallBackEvent;
    sCallBackEvent.pZPSevent = psStackEvent;

    /*
     * If the 1 second tick timer has expired, restart it and pass
     * the event on to ZCL
     */
    //DBG_vPrintf(TRACE_ZCL, "ZCL TASK\n");

    DBG_vPrintf(TRACE_ZCL, "ZCL_Task got event %d\r\n",psStackEvent->eType);

    switch(psStackEvent->eType)
    {

    case ZPS_EVENT_APS_DATA_INDICATION:
        DBG_vPrintf(TRACE_ZCL, "\nDATA: SEP=%d DEP=%d Profile=%04x Cluster=%04x\n",
                    psStackEvent->uEvent.sApsDataIndEvent.u8SrcEndpoint,
                    psStackEvent->uEvent.sApsDataIndEvent.u8DstEndpoint,
                    psStackEvent->uEvent.sApsDataIndEvent.u16ProfileId,
                    psStackEvent->uEvent.sApsDataIndEvent.u16ClusterId);
        break;

    case ZPS_EVENT_APS_DATA_CONFIRM:
        DBG_vPrintf(TRACE_ZCL, "\nCFM: SEP=%d DEP=%d Status=%d\n",
                    psStackEvent->uEvent.sApsDataConfirmEvent.u8SrcEndpoint,
                    psStackEvent->uEvent.sApsDataConfirmEvent.u8DstEndpoint,
                    psStackEvent->uEvent.sApsDataConfirmEvent.u8Status);

        break;

    case ZPS_EVENT_APS_DATA_ACK:
        DBG_vPrintf(TRACE_ZCL, "\nACK: SEP=%d DEP=%d Profile=%04x Cluster=%04x\n",
                    psStackEvent->uEvent.sApsDataAckEvent.u8SrcEndpoint,
                    psStackEvent->uEvent.sApsDataAckEvent.u8DstEndpoint,
                    psStackEvent->uEvent.sApsDataAckEvent.u16ProfileId,
                    psStackEvent->uEvent.sApsDataAckEvent.u16ClusterId);
        break;

    case ZPS_EVENT_APS_INTERPAN_DATA_CONFIRM:

        if(psStackEvent->uEvent.sApsInterPanDataConfirmEvent.u8Status)
        {
            //sCommissionEvent.eType = BDB_E_ZCL_EVENT_TL_NOACK;
        }
        else
        {
            //sCommissionEvent.eType = BDB_E_ZCL_EVENT_TL_ACK;
        }
        //BDB_vZclEventHandler(&sCommissionEvent);
        break;

    default:
        break;
    }

    sCallBackEvent.eEventType = E_ZCL_CBET_ZIGBEE_EVENT;
    vZCL_EventHandler(&sCallBackEvent);

}


/****************************************************************************
 *
 * NAME: APP_ZCL_cbGeneralCallback
 *
 * DESCRIPTION:
 * General callback for ZCL events
 *
 * RETURNS:
 * void
 *
 ****************************************************************************/
PRIVATE void APP_ZCL_cbGeneralCallback(tsZCL_CallBackEvent *psEvent)
{
#if TRUE == TRACE_ZCL
    switch (psEvent->eEventType)
    {
        case E_ZCL_CBET_LOCK_MUTEX:
//            //DBG_vPrintf(TRACE_ZCL, "EVT: Lock Mutex\r\n");
            break;

        case E_ZCL_CBET_UNLOCK_MUTEX:
//            DBG_vPrintf(TRACE_ZCL, "EVT: Unlock Mutex\r\n");
            break;

        case E_ZCL_CBET_UNHANDLED_EVENT:
            //DBG_vPrintf(TRACE_ZCL, "EVT: Unhandled Event\r\n");
            break;

        case E_ZCL_CBET_READ_ATTRIBUTES_RESPONSE:
            DBG_vPrintf(TRACE_ZCL, "EVT: Read attributes response\r\n");
            break;

        case E_ZCL_CBET_READ_REQUEST:
            //DBG_vPrintf(TRACE_ZCL, "EVT: Read request\r\n");
            break;

        case E_ZCL_CBET_DEFAULT_RESPONSE:
            //DBG_vPrintf(TRACE_ZCL, "EVT: Default response\r\n");
            break;

        case E_ZCL_CBET_ERROR:
            //DBG_vPrintf(TRACE_ZCL, "EVT: Error\r\n");
            break;

        case E_ZCL_CBET_TIMER:
            break;

        case E_ZCL_CBET_ZIGBEE_EVENT:
            //DBG_vPrintf(TRACE_ZCL, "EVT: ZigBee\r\n");
            break;

        case E_ZCL_CBET_CLUSTER_CUSTOM:
            //DBG_vPrintf(TRACE_ZCL, "EP EVT: Custom\r\n");
            break;

        default:
            //DBG_vPrintf(TRACE_ZCL, "Invalid event type\r\n");
            break;
    }
#endif
}


/****************************************************************************
 *
 * NAME: APP_ZCL_cbEndpointCallback
 *
 * DESCRIPTION:
 * Endpoint specific callback for ZCL events
 *
 * RETURNS:
 * void
 *
 ****************************************************************************/
PRIVATE void APP_ZCL_cbEndpointCallback(tsZCL_CallBackEvent *psEvent)
{
    uint32 i;

    DBG_vPrintf(TRACE_ZCL, "\nEntering cbZCL_EndpointCallback");

    switch (psEvent->eEventType)
    {
        case E_ZCL_CBET_LOCK_MUTEX:
        case E_ZCL_CBET_UNLOCK_MUTEX:
        case E_ZCL_CBET_UNHANDLED_EVENT:
        case E_ZCL_CBET_READ_ATTRIBUTES_RESPONSE:
        case E_ZCL_CBET_READ_REQUEST:
        case E_ZCL_CBET_DEFAULT_RESPONSE:
        case E_ZCL_CBET_ERROR:
        case E_ZCL_CBET_TIMER:
        case E_ZCL_CBET_ZIGBEE_EVENT:
        {
            /* Do nothing */
            ;
            // DBG_vPrintf(TRACE_ZCL, "EP EVT:No action\r\n");
        }
        break;

        case E_ZCL_CBET_READ_INDIVIDUAL_ATTRIBUTE_RESPONSE:
        {
            DBG_vPrintf(TRACE_REMOTE_TASK, " Read Attrib Rsp %d %02x\n", psEvent->uMessage.sIndividualAttributeResponse.eAttributeStatus,
                    *((uint8*)psEvent->uMessage.sIndividualAttributeResponse.pvAttributeData));

            for ( i=0; i<sEndpointTable.u8NumRecords; i++) {
                if (sEndpointTable.asEndpointRecords[i].u16NwkAddr == psEvent->pZPSevent->uEvent.sApsDataIndEvent.uSrcAddress.u16Addr) {
                    sEndpointTable.au8PingCount[i] = 0;
                    DBG_vPrintf(TRACE_LIGHT_AGE, "Reset Ping Count for %04x\n", sEndpointTable.asEndpointRecords[i].u16NwkAddr);
                    break;
                }
            }

            uint8 u8Seq;
            tsZCL_Address sAddress;
            sAddress.eAddressMode = E_ZCL_AM_SHORT_NO_ACK;
            sAddress.uAddress.u16DestinationAddress = psEvent->pZPSevent->uEvent.sApsDataIndEvent.uSrcAddress.u16Addr;
            tsCLD_Identify_IdentifyRequestPayload sPayload;
            sPayload.u16IdentifyTime = 0x0003;

            eCLD_IdentifyCommandIdentifyRequestSend(
                            sDeviceTable.asDeviceRecords[0].u8Endpoint,
                            psEvent->pZPSevent->uEvent.sApsDataIndEvent.u8SrcEndpoint,
                            &sAddress,
                            &u8Seq,
                            &sPayload);
        }
        break;

        case E_ZCL_CBET_CLUSTER_CUSTOM:
        {
            DBG_vPrintf(TRACE_ZCL, "EP EVT: Custom %04x\r\n", psEvent->uMessage.sClusterCustomMessage.u16ClusterId);

            switch (psEvent->uMessage.sClusterCustomMessage.u16ClusterId)
            {
                #ifdef CLD_OTA
                    case OTA_CLUSTER_ID:
                    {
                        tsOTA_CallBackMessage *psCallBackMessage = (tsOTA_CallBackMessage *)psEvent->uMessage.sClusterCustomMessage.pvCustomData;
                        vHandleAppOtaClient(psCallBackMessage);
                    }
                    break;
                #endif

                case GENERAL_CLUSTER_ID_BASIC:
                {
                    tsCLD_BasicCallBackMessage *psCallBackMessage = (tsCLD_BasicCallBackMessage*)psEvent->uMessage.sClusterCustomMessage.pvCustomData;
                    if (psCallBackMessage->u8CommandId == E_CLD_BASIC_CMD_RESET_TO_FACTORY_DEFAULTS )
                    {
                        DBG_vPrintf(TRACE_RESET, "Basic Factory Reset Received\n");
                        vApp_ZCL_ResetDeviceStructure();
                        eApp_ZLO_RegisterEndpoint(&APP_ZCL_cbEndpointCallback);
                        vAPP_ZCL_DeviceSpecific_Init();
                        #ifdef CLD_OTA
                            vAppInitOTA();
                        #endif
                    }
                }
                break;

                case GENERAL_CLUSTER_ID_IDENTIFY:
                {
                    tsCLD_IdentifyCallBackMessage *psCallBackMessage = (tsCLD_IdentifyCallBackMessage*)psEvent->uMessage.sClusterCustomMessage.pvCustomData;
                    DBG_vPrintf(TRACE_ZCL, "- for identify cluster\r\n");
                    if(psEvent->psClusterInstance->bIsServer == FALSE)
                    {
                        // client commands
                        if (psCallBackMessage->u8CommandId == E_CLD_IDENTIFY_CMD_IDENTIFY)
                        {
                            DBG_vPrintf(TRACE_ZCL, "\nJP E_CLD_IDENTIFY_CMD_IDENTIFY");
                            tsBDB_ZCLEvent  sBDBZCLEvent;
                            /* provide callback to BDB handler for identify query response on initiator*/

                            DBG_vPrintf(TRACE_ZCL, "\nCallBackBDB");
                            sBDBZCLEvent.eType = BDB_E_ZCL_EVENT_IDENTIFY_QUERY;
                            sBDBZCLEvent.psCallBackEvent = psEvent;
                            BDB_vZclEventHandler(&sBDBZCLEvent);
                        }
                    }
                    else
                    {
                        // Server commands
                        if (psCallBackMessage->u8CommandId == E_CLD_IDENTIFY_CMD_IDENTIFY)
                        {
                            vHandleIdentifyRequest(sRemote.sIdentifyServerCluster.u16IdentifyTime);
                        }
                        else if (psCallBackMessage->u8CommandId == E_CLD_IDENTIFY_CMD_TRIGGER_EFFECT)
                        {
                            vStartEffect( psCallBackMessage->uMessage.psTriggerEffectRequestPayload->eEffectId);
                        }
                    }
                }
                break;

                case GENERAL_CLUSTER_ID_GROUPS:
                {
                    DBG_vPrintf(TRACE_ZCL, "- for groups cluster\r\n");
                }
                break;

                case 0x1000:
                {
                    DBG_vPrintf(TRACE_ZCL, "\n    - for 0x1000");
                    if (psEvent->pZPSevent->eType == ZPS_EVENT_APS_INTERPAN_DATA_INDICATION &&  psEvent->pZPSevent->uEvent.sApsInterPanDataIndEvent.u16ProfileId == ZLL_PROFILE_ID)
                    {
                        tsBDB_ZCLEvent sEvent;
                        sEvent.eType = BDB_E_ZCL_EVENT_TL_IPAN_MSG;
                        sEvent.psCallBackEvent = psEvent;
                        BDB_vZclEventHandler(&sEvent);
                    }
                    else if (psEvent->pZPSevent->eType == ZPS_EVENT_APS_DATA_INDICATION && psEvent->pZPSevent->uEvent.sApsDataIndEvent.u16ProfileId == HA_PROFILE_ID)
                    {
                        APP_ZCL_cbZllUtilityCallback(psEvent);
                    }
                }
                break;

                default:
                {
                    DBG_vPrintf(TRACE_ZCL, "- for unknown cluster %d\r\n", psEvent->uMessage.sClusterCustomMessage.u16ClusterId);
                }
                break;
            }
        }
        break;

        case E_ZCL_CBET_CLUSTER_UPDATE:
        {
            if (psEvent->psClusterInstance->psClusterDefinition->u16ClusterEnum == GENERAL_CLUSTER_ID_IDENTIFY)
            {
                if (psEvent->psClusterInstance->bIsServer == TRUE)
                {
                    /* Do nothing */
                    ;
                }
            }
        }
        break;

        default:
        {
            DBG_vPrintf(TRACE_ZCL, "EP EVT: Invalid event type\r\n");
        }
        break;

    }
}



PRIVATE void APP_ZCL_cbZllUtilityCallback(tsZCL_CallBackEvent *psEvent)
{
    APP_tsEvent   sEvent;

    DBG_vPrintf(TRACE_REMOTE_TASK|TRUE, "\nRx Util Cmd %02x",
            ((tsCLD_ZllCommissionCustomDataStructure*)psEvent->psClusterInstance->pvEndPointCustomStructPtr)->sCallBackMessage.u8CommandId);

    switch (((tsCLD_ZllCommissionCustomDataStructure*)psEvent->psClusterInstance->pvEndPointCustomStructPtr)->sCallBackMessage.u8CommandId ) {
        case E_CLD_UTILITY_CMD_ENDPOINT_INFO:
            sEvent.eType = APP_E_EVENT_EP_INFO_MSG;
            sEvent.uEvent.sEpInfoMsg.u16SrcAddr = psEvent->pZPSevent->uEvent.sApsDataIndEvent.uSrcAddress.u16Addr;
            memcpy(&sEvent.uEvent.sEpInfoMsg.sPayload,
                    ((tsCLD_ZllCommissionCustomDataStructure*)psEvent->psClusterInstance->pvEndPointCustomStructPtr)->sCallBackMessage.uMessage.psEndpointInfoPayload,
                    sizeof(tsCLD_ZllUtility_EndpointInformationCommandPayload));
            ZQ_bQueueSend(&APP_msgAppEvents, &sEvent);
            break;

        case E_CLD_UTILITY_CMD_GET_ENDPOINT_LIST_REQ_RSP:
            DBG_vPrintf(TRACE_REMOTE_TASK, "\ngot ep list");
            sEvent.eType = APP_E_EVENT_EP_LIST_MSG;
            sEvent.uEvent.sEpListMsg.u8SrcEp = psEvent->pZPSevent->uEvent.sApsDataIndEvent.u8SrcEndpoint;
            sEvent.uEvent.sEpListMsg.u16SrcAddr = psEvent->pZPSevent->uEvent.sApsDataIndEvent.uSrcAddress.u16Addr;
            memcpy(&sEvent.uEvent.sEpListMsg.sPayload,
                    ((tsCLD_ZllCommissionCustomDataStructure*)psEvent->psClusterInstance->pvEndPointCustomStructPtr)->sCallBackMessage.uMessage.psGetEndpointListRspPayload,
                    sizeof(tsCLD_ZllUtility_GetEndpointListRspCommandPayload));
            ZQ_bQueueSend(&APP_msgAppEvents, &sEvent);
            break;
        case E_CLD_UTILITY_CMD_GET_GROUP_ID_REQ_RSP:
            DBG_vPrintf(TRACE_REMOTE_TASK, "\ngot group list");
            sEvent.eType = APP_E_EVENT_GROUP_LIST_MSG;
            sEvent.uEvent.sGroupListMsg.u8SrcEp = psEvent->pZPSevent->uEvent.sApsDataIndEvent.u8SrcEndpoint;
            sEvent.uEvent.sGroupListMsg.u16SrcAddr = psEvent->pZPSevent->uEvent.sApsDataIndEvent.uSrcAddress.u16Addr;
            memcpy(&sEvent.uEvent.sGroupListMsg.sPayload,
                    ((tsCLD_ZllCommissionCustomDataStructure*)psEvent->psClusterInstance->pvEndPointCustomStructPtr)->sCallBackMessage.uMessage.psGetGroupIdRspPayload,
                    sizeof(tsCLD_ZllUtility_GetGroupIdRspCommandPayload));
            ZQ_bQueueSend(&APP_msgAppEvents, &sEvent);
            break;
    }
}

/****************************************************************************
 *
 * NAME: vStartEffect
 *
 * DESCRIPTION:
 * ZLO Device Specific identify effect set up
 *
 * PARAMETER: void
 *
 * RETURNS: void
 *
 ****************************************************************************/
PRIVATE void vStartEffect(uint8 u8Effect)
{
    switch (u8Effect) {
        case E_CLD_IDENTIFY_EFFECT_BLINK:
            sRemote.sIdentifyServerCluster.u16IdentifyTime = 2;
            break;

        case E_CLD_IDENTIFY_EFFECT_BREATHE:
            sRemote.sIdentifyServerCluster.u16IdentifyTime = 17;
            break;

        case E_CLD_IDENTIFY_EFFECT_OKAY:
            sRemote.sIdentifyServerCluster.u16IdentifyTime = 3;
            break;

        case E_CLD_IDENTIFY_EFFECT_CHANNEL_CHANGE:
            sRemote.sIdentifyServerCluster.u16IdentifyTime = 9;
            break;

        case E_CLD_IDENTIFY_EFFECT_FINISH_EFFECT:
        case E_CLD_IDENTIFY_EFFECT_STOP_EFFECT:
            sRemote.sIdentifyServerCluster.u16IdentifyTime = 1;
            break;
    }
    vHandleIdentifyRequest( sRemote.sIdentifyServerCluster.u16IdentifyTime);
}

/****************************************************************************/
/***        END OF FILE                                                   ***/
/****************************************************************************/
