/*****************************************************************************
 * Copyright  2018 NXP
 * All rights reserved
 *

 * SPDX-License-Identifier: LA_OPT_BASE_LICENSE v24 May 2018
 *****************************************************************************/
/*****************************************************************************
 *
 * MODULE:            JN-AN-1245
 *
 * COMPONENT:          EH_IEEE_802154_Switch.c
 *
 * DESCRIPTION:        ZigBee GP switch main file
 *
 ****************************************************************************
 *
 * This software is owned by NXP B.V. and/or its supplier and is protected
 * under applicable copyright laws. All rights are reserved. We grant You,
 * and any third parties, a license to use this software solely and
 * exclusively on NXP products [NXP Microcontrollers such as JN5168, JN5169,
 * JN5179, JN5189].
 * You, and any third parties must reproduce the copyright and warranty notice
 * and any other legend of ownership on each copy or partial copy of the
 * software.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright NXP B.V. 2016-2018. All rights reserved
 *
 ***************************************************************************/

/****************************************************************************/
/***        Include files                                                 ***/
/****************************************************************************/
#include "jendefs.h"
#include "string.h"
#include <dbg.h>
#include "MMAC.h"
#include "EH_IEEE_802154_Switch.h"
#include "EH_Switch_Configurations.h"
#include "EH_IEEE_Features.h"
#include "EH_IEEE_Commands.h"
#include "EH_Button.h"
#include "app_nvm.h"
#ifndef K32W1480_SERIES
#include "fsl_power.h"
#include "fsl_gpio.h"
#include "DebugExceptionHandlers_jn518x.h"
#include "radio.h"
#endif

#if (defined OM15082) || (defined K32W1480_SERIES)
#include "app_main.h"
#include "ZTimer.h"
#include "fsl_os_abstraction.h"
#include "fsl_os_abstraction_bm.h"
#endif
#ifdef K32W1480_SERIES
#include "SecLib.h"
#include "RNG_Interface.h"
#include "fwk_platform.h"
#include "fwk_platform_ics.h"
#include "fsl_component_mem_manager.h"
#include "fsl_component_timer_manager.h"
#include "pwrm.h"
#include "PhyInterface.h"
#include "smac_interface.h"
#include "fsl_wuu.h"
#include "smac.h"
#include "fsl_wdog32.h"
#include "fsl_cmc.h"
#include "pin_mux.h"
#endif
#include "mac_sap.h"
#include "mac_vs_sap.h"

/****************************************************************************/
/***        Macro Definitions                                             ***/
/****************************************************************************/
/* Required to set BrownOut */
#if 0
#define VBO_MASK            0x0000000eUL
#define SYSCON_SYS_IM_ADDR  0x0200000cUL
#define SYSCON_VBOCTRL_ADDR 0x02000044UL
#endif

/* DIO Used for Cap Switching */
#define DIO_1 (0x00000002)
#define DIO_2 (0x00000004)
#define DIO_3 (0x00000008)
/* Mask for all Cap switching buttons */
#define DIO_CAP_MASK (DIO_1|DIO_2|DIO_3)

#define BUTTON_PRESSED       0x00               /*No DIO set */
#define BUTTON_RELEASED      DIO_8              /*DIO 8 set when button released*/

#ifdef  K32W1480_SERIES
static uint8_t gau8RxDataBuffer[gMaxSmacSDULength_c  + sizeof(rxPacket_t)];
uint8_t gau8TxDataBuffer[gMaxSmacSDULength_c  + sizeof(txPacket_t)];

txPacket_t * gAppTxPacket;
static rxPacket_t * gAppRxPacket;
WDOG_Type *wdog32_base = WDOG0;
#endif

/****************************************************************************/
/***        Type Definitions                                              ***/
/****************************************************************************/
/* Brown Out */
/*
typedef enum
{
    E_BO_TRIP_1V95 = 0,
    E_BO_TRIP_2V0  = 2,
    E_BO_TRIP_2V1  = 4,
    E_BO_TRIP_2V2  = 6,
    E_BO_TRIP_2V3  = 8,
    E_BO_TRIP_2V4  = 10,
    E_BO_TRIP_2V7  = 12,
    E_BO_TRIP_3V0  = 14
} teBrownOutTripVoltage;
*/
/****************************************************************************/
/***        Local Function Prototypes                                     ***/
/****************************************************************************/
#ifdef OM15082
PUBLIC void APP_vInitResources(void);
#endif
volatile bool_t bEnableSleep = TRUE;
/****************************************************************************/
/***        Exported Variables                                            ***/
/****************************************************************************/

/****************************************************************************/
/***        Local Variables                                               ***/
/****************************************************************************/

gpio_pin_config_t led_config = {
    kGPIO_DigitalOutput, 0,
};

/* bTxComplete is used to flag TX completion from ISR to main code */
volatile bool_t bTxComplete;

/*bRxComplete is used to flag RX completion from ISR to main code */
volatile bool_t bRxComplete;

extern const uint8_t gUseRtos_c;
/****************************************************************************/
/***        Exported Functions                                            ***/
/****************************************************************************/
extern void OSA_TimeInit(void);

#ifdef K32WMCM_APP_BUILD
/* Must be called before zps_eAplAfInit() */
void APP_SetHighTxPowerMode();

/* Must be called after zps_eAplAfInit() */
void APP_SetMaxTxPower();

#undef HIGH_TX_PWR_LIMIT
#define HIGH_TX_PWR_LIMIT 15	/* dBm */
/* High Tx power */
void APP_SetHighTxPowerMode()
{
	if (CHIP_IS_HITXPOWER_CAPABLE())
		vMMAC_SetTxPowerMode(TRUE);
}

void APP_SetMaxTxPower()
{
	if (CHIP_IS_HITXPOWER_CAPABLE())
		ZPS_eMacPlmeSet(PHY_PIB_ATTR_TX_POWER, HIGH_TX_PWR_LIMIT);
}
#endif

#ifdef K32WMCM_APP_BUILD
#define MAX_TX_POWER 15
#else
#define MAX_TX_POWER 10
#endif
/****************************************************************************
 *
 * NAME: vSendMACFrame
 *
 * DESCRIPTION:
 * Transmits MAC frame and updates persistent data
 *
 * RETURNS:
 * void
 *
 ****************************************************************************/

PUBLIC void vSendMACFrame( bool bUpdatePersistentData)
{
    uint8 u8TxRepeats = 0;
    if(bUpdatePersistentData)
    {
        /* update persistence data and button press count*/
#ifdef GPD_SUPPORT_PERSISTENT_DATA
        bAPP_NVMWriteBlock(  sizeof(tGPD_PersistantData),
                             (uint8 *)&sGPDPersistentData);
#endif
    }

    /* Set Dedicated single channel */
#ifndef K32W148EVK
        vMMAC_SetChannelAndPower(sGPDPersistentData.u8Channel, MAX_TX_POWER);
#else
        MLMESetChannelRequest(sGPDPersistentData.u8Channel);
        MLMEPAOutputAdjust(MAX_TX_POWER);
#endif
    do
    {
        DBG_vPrintf(DBG_DEVICE_SWITCH, "\r\n vSendMACFrame channel = %d  ", sGPDPersistentData.u8Channel );

        /* Set transmit flag to false */
        bTxComplete = FALSE;
#ifndef K32W148EVK
        /* Start Transmit with no CCA or ACK */
        vMMAC_StartMacTransmit(&sGPD_MACFrame, E_MMAC_TX_START_NOW
                                           | E_MMAC_TX_NO_AUTO_ACK
                                           | E_MMAC_TX_NO_CCA);
#else
        gAppTxPacket->smacHeader.frameControl = sGPD_MACFrame.u16FCF;
  	 gAppTxPacket->smacHeader.panId        = sGPD_MACFrame.u16DestPAN;
        gAppTxPacket->smacHeader.seqNo        = sGPD_MACFrame.u8SequenceNum;
        //gAppTxPacket->smacHeader.srcAddr      = sGPD_MACFrame.uSrcAddr.u16Short;
        gAppTxPacket->smacHeader.destAddr     = sGPD_MACFrame.uDestAddr.u16Short;
        gAppTxPacket->u8DataLength = sGPD_MACFrame.u8PayloadLength;

        memcpy(gAppTxPacket->smacPdu.smacPdu, sGPD_MACFrame.uPayload.au8Byte, sGPD_MACFrame.u8PayloadLength);
        MCPSDataRequest(gAppTxPacket);
#endif
        /* While call-back not reset flag */
#ifndef K32W148EVK
        while (bTxComplete != TRUE)
#else
        uint32 u32StartTime = OSA_TimeGetMsec();
        while ((bTxComplete != TRUE)
           && ((OSA_TimeGetMsec() - u32StartTime) < 250))
#endif
        {
            /*
             * suspends CPU operation when the system is idle or puts the device to
             * sleep if there are no activities in progress
             */
#ifndef K32W148EVK
            //POWER_SetLowPower( PM_SLEEP_0 , &pmconfig);
        POWER_EnterSleep();
#else
        //TODO : sleep?
	 //PWR_TryEnterLowPower();
#endif
        }
        /* add one to repeat value */
        u8TxRepeats++;
    /* Keep transmitting until counter reached or power runs out
     * or if powered by mains for a max of 20 packets */
    } while (u8TxRepeats < sGPDPersistentData.u8NoOfTransPerChannel);

}
/****************************************************************************
 *
 * NAME: vEnableRxAndReceive
 *
 * DESCRIPTION:
 * Turns on Rx and receives data
 *
 * RETURNS:
 * void
 *
 ****************************************************************************/
#if defined( GPD_RX_AFTER_TX) || defined(GPD_RX_ENABLE)
void vEnableRxAndReceive(void)
{
#ifndef K32W1480_SERIES
#ifdef OM15082
    uint32 u32StartTime;
#endif

    /* MAC clock gives time in 16us resolution */
    /* keep time as 10 milli seconds  */

    bRxComplete = FALSE;
    /* Set time corresponding to MAC clock */
    vMMAC_RadioToOffAndWait();
    vMMAC_StartMacReceive(&sGPD_MACReceivedFrame, E_MMAC_RX_START_NOW
                            |E_MMAC_RX_NO_AUTO_ACK
                            );
#ifndef OM15082
/* While call-back not reset flag */
    while( bRxComplete == FALSE)
    {
        //POWER_SetLowPower( PM_SLEEP_0 , &pmconfig);
    POWER_EnterSleep();
    }
    vHandleRxMacFrame();

#else
    /* Don't use ZTIMER or call into ZTIMER_vTask(). We are already in a
     * callback from ZTIMER_vTask() and calling it from here messes up the
     * timing spectacularly. So use OSA time function directly. */
    u32StartTime = OSA_TimeGetMsec();

    while (   (FALSE == bRxComplete)
           && ((OSA_TimeGetMsec() - u32StartTime) < 250)
          )
    {
        /* Tight loop */
    }

    if(bRxComplete)
    {
        /* Process Received frame */
        DBG_vPrintf(DBG_DEVICE_SWITCH, "\r\n bRxComplete  vHandleRxMacFrame " );
        vHandleRxMacFrame();
    }
    else
    {
        DBG_vPrintf(DBG_DEVICE_SWITCH, "\r\n MAC receive Timer Expired " );
    }

    vMMAC_RadioOff();
#endif
#else /*K32W1480_SERIES*/
     uint32 u32StartTime;
     bRxComplete = FALSE;
     MLMETXDisableRequest();
     MLMERXEnableRequest(gAppRxPacket, 0);
     u32StartTime = OSA_TimeGetMsec();
     while (   (FALSE == bRxComplete)
           && ((OSA_TimeGetMsec() - u32StartTime) < 250)
          )
     {
         /* Tight loop */
     }
     DBG_vPrintf(TRUE, "bRxComplete : %d\n",bRxComplete);
     if(bRxComplete)
         vHandleRxMacFrame();
     MLMERXDisableRequest();
#endif
}
#endif
#ifndef K32W1480_SERIES
/****************************************************************************/
/***        Local Functions                                               ***/
/****************************************************************************/
/****************************************************************************
 *
 * NAME: vInterruptFired
 *
 * DESCRIPTION:
 * ISR for MAC RX/TX interrupts
 *
 * RETURNS:
 * void
 *
 ****************************************************************************/
PRIVATE void vInterruptFired(uint32 u32InterruptBitmap)
{
    /* Is the interrupt for Transmit completed */
    if (u32InterruptBitmap & E_MMAC_INT_TX_COMPLETE)
    {
        bTxComplete = TRUE;
    }
    else
    {
        /* Is the interrupt for Rx completed */
        if((u32InterruptBitmap & E_MMAC_INT_RX_COMPLETE))
        {
            bRxComplete = TRUE;
        }
    }
}
#endif
#ifdef K32W1480_SERIES
/*(Management) Sap handler for managing timeout indication and ED confirm */
smacErrors_t smacToAppMlmeSap(smacToAppMlmeMessage_t* pMsg, instanceId_t instance)
{
    switch(pMsg->msgType)
    {
    case gMlmeEdCnf_c:
        break;
    case gMlmeCcaCnf_c:
      
        break;
    case gMlmeTimeoutInd_c:
        
	break;
    default:
        break;
    }
    MEM_BufferFree(pMsg);
    return gErrorNoError_c;
}
/*(Data) Sap handler for managing data confirm and data indication */
smacErrors_t smacToAppMcpsSap(smacToAppDataMessage_t* pMsg, instanceId_t instance)
{
    switch(pMsg->msgType)
    {
    case gMcpsDataInd_c:
        if(pMsg->msgData.dataInd.pRxPacket->rxStatus == rxSuccessStatus_c)
        {
            DBG_vPrintf(TRUE, "received u8DataLength : %d\n",pMsg->msgData.dataInd.pRxPacket->u8DataLength);
	     /*pMsg->msgData.dataInd.pRxPacket + 4  moves the pointer to smacHeader*/
	     /*SMAC header data has two extra bytes compare to GP packets which doesn't have source address in its mac header, subtract them to get the real payload*/
            memcpy(sGPD_MACReceivedFrame.uPayload.au8Byte, (uint8_t *)(pMsg->msgData.dataInd.pRxPacket) + 11, pMsg->msgData.dataInd.pRxPacket->u8DataLength);
	     sGPD_MACReceivedFrame.u8PayloadLength = pMsg->msgData.dataInd.pRxPacket->u8DataLength;
        }
	  bRxComplete = TRUE;
        break;
    case gMcpsDataCnf_c:
	  if(pMsg->msgData.dataCnf.status == gErrorNoAck_c) {
			//bTxNoAck = TRUE;
	  }
	  bTxComplete = TRUE;
        break;
    default:
        break;
    }

    MEM_BufferFree(pMsg);
    return gErrorNoError_c;
}
#endif

/****************************************************************************
 *
 * NAME: AppWarmStart
 *
 * DESCRIPTION:
 * Entry point for application from boot loader. Simply jumps to AppColdStart
 * as, in this instance, application will never warm start.
 *
 * RETURNS:
 * void
 *
 ****************************************************************************/
PUBLIC void vAppMain(void)
{
    bool bClearPDM;

#ifndef K32W1480_SERIES
    /* Initialise exception handlers for debugging */
    vDebugExceptionHandlersInitialise();
#endif

#ifdef K32W1480_SERIES
    /*Enable Watchdog*/
    wdog32_config_t config;
    WDOG32_GetDefaultConfig(&config);
    config.prescaler = kWDOG32_ClockPrescalerDivide256;
    /*default clock is 32K, divide by 256 = 125*/
    config.timeoutValue = 125*10; /*10s*/
    WDOG32_Init(wdog32_base, &config);
    WDOG32_Refresh(wdog32_base);
#endif

#if (defined OM15082) || (defined K32W148EVK)
    /* Initialize DIO on OM15082 switch*/
    APP_vInitResources();
    bClearPDM =
#endif
    bButtonInitialize();

#ifdef K32WMCM_APP_BUILD
    APP_SetHighTxPowerMode();
#endif

#ifndef K32W1480_SERIES
    /* Provide fake temperature to radio to ensure calibration settings are saved */
    vRadio_Temp_Update(40);
#endif

#ifndef K32W1480_SERIES
    /* Enable MAC, and enable interrupts */
    vMMAC_Enable();
    vMMAC_ConfigureRadio();
    vMMAC_EnableInterrupts(vInterruptFired);
#else
    Phy_Init();
    gAppTxPacket = (txPacket_t*)gau8TxDataBuffer;   //Map TX packet to buffer
    gAppRxPacket = (rxPacket_t*)gau8RxDataBuffer;   //Map Rx packet to buffer
    gAppRxPacket->u8MaxDataLength = gMaxSmacSDULength_c;
    //Initialise SMAC
    InitSmac();
    //Tell SMAC who to call when it needs to pass a message to the application thread.
    Smac_RegisterSapHandlers((SMAC_APP_MCPS_SapHandler_t)smacToAppMcpsSap,(SMAC_APP_MLME_SapHandler_t)smacToAppMlmeSap,0);
    SMACFillHeader(&(gAppTxPacket->smacHeader), gBroadcastAddress_c);  
    PhyPpSetPromiscuous(TRUE);
#endif
	
#ifdef K32WMCM_APP_BUILD
    APP_SetMaxTxPower();
#endif

#if (defined OM15082) || (defined K32W148EVK)
    if(bClearPDM)
    {
        vClearPersistentData(TRUE);
    }
#else

    #ifdef GP_LEVEL_CONTROL
        vHandleLevelControlButtonPress();
    #else
        vHandleEHButtonPress();
    #endif
#endif

}

#ifndef K32W148EVK
/****************************************************************************
 *
 * NAME: APP_vSetUpHardware
 *
 * DESCRIPTION:
 * Set up interrupts
 *
 * RETURNS:
 * void
 *
 ****************************************************************************/
PUBLIC void APP_vSetUpHardware(void)
{
    /* Enable DMA access to RAM (assuming default configuration and MAC
     * buffers not in first block of RAM) */
    *(volatile uint32 *)0x40001000 = 0xE000F733;
    *(volatile uint32 *)0x40001004 = 0x109;
    /* Board pin, clock, debug console init */
    BOARD_InitHardware();

    /* Initialise output LED GPIOs */
    GPIO_PinInit(GPIO, APP_BOARD_GPIO_PORT, APP_BASE_BOARD_LED1_PIN, &led_config);
    GPIO_PinInit(GPIO, APP_BOARD_GPIO_PORT, APP_BASE_BOARD_LED1_PIN, &led_config);

    SYSCON -> MAINCLKSEL       = 3;  /* 32 M FRO */
    SystemCoreClockUpdate();
    OSA_TimeInit();
    /* Initialise buttons;
     */
    bButtonInitialize();

}
#endif
#ifdef K32W148EVK
void PWR_vWakeUpConfig(uint8_t u8PinIndex)
{
     wuu_external_wakeup_pin_config_t wakeupButtonConfig;
     wakeupButtonConfig.edge  = kWUU_ExternalPinAnyEdge;
     wakeupButtonConfig.event = kWUU_ExternalPinInterrupt;
     wakeupButtonConfig.mode  = kWUU_ExternalPinActiveDSPD;
     WUU_SetExternalWakeUpPinsConfig(WUU0, u8PinIndex, &wakeupButtonConfig);
}

#ifdef APP_LOW_POWER_API
static void PreSleep(void)
#else
PWRM_CALLBACK(PreSleep)
#endif
{
     DBG_vPrintf(TRUE, "\nAPP: Sleep\n");
     PWR_vWakeUpConfig(APP_BUTTONS_BUTTON_0_WAKE_INDEX);
#ifndef LOW_CURRENT_TEST_MODE
     PWR_vWakeUpConfig(APP_BUTTONS_BUTTON_1_WAKE_INDEX);
     PWR_vWakeUpConfig(APP_BUTTONS_BUTTON_2_WAKE_INDEX);
     PWR_vWakeUpConfig(APP_BUTTONS_BUTTON_3_WAKE_INDEX); 
     PWR_vWakeUpConfig(APP_BUTTONS_BUTTON_4_WAKE_INDEX);
#endif
    extern void BOARD_SetPinsForPowerMode(void);
    BOARD_SetPinsForPowerMode();

    //TODO : low power for SMAC doesn't work
    //PHY_Disable();
    //XCVR_Reset();
    //XCVR_Deinit();
}

#ifdef APP_LOW_POWER_API
static void Wakeup(void)
#else
PWRM_CALLBACK(Wakeup)
#endif
{		
     //BOARD_InitHardware();
     BOARD_InitPinLPUART1_RX();
     BOARD_InitPinLPUART1_TX();
     DBG_vPrintf(TRUE, "\nAPP: before Woken up (CB) %d\n",PWRM_u16GetActivityCount());
     /*TODO : activity count is 1 after wakeup !!!*/
     //while(PWRM_u16GetActivityCount())
	//PWRM_eFinishActivity();
        
     //APP_vInitResources();
     bButtonInitialize();
     ZTIMER_eStart(u8TimerButtonScan, ZTIMER_TIME_MSEC(10));
}
void vAppRegisterPWRCallbacks(void)
{
#ifdef APP_LOW_POWER_API
    PWR_RegisterLowPowerEnterCallback(PreSleep);
    PWR_RegisterLowPowerExitCallback(Wakeup);
#else
    PWRM_vRegisterPreSleepCallback(PreSleep);
    PWRM_vRegisterWakeupCallback(Wakeup);
#endif
}
#endif
/****************************************************************************
 *
 * NAME: APP_vMainLoop
 *
 * DESCRIPTION:
 * Main application loop
 *
 * RETURNS:
 * void
 *
 ****************************************************************************/
void main_task (uint32_t parameter)
{
    /* e.g. osaEventFlags_t ev; */
    static uint8_t initialized = FALSE;

    if(!initialized)
    {
        /* place initialization code here... */
        /*myEventId = OSA_EventCreate(TRUE);*/
        initialized = TRUE;
#ifdef K32W1480_SERIES
        PLATFORM_SwitchToOsc32k();
        PLATFORM_InitTimerManager();
	 PWR_vColdStart();
	 //TODO : deep sleep doesn't work
        //DBG_vPrintf(DBG_DEVICE_SWITCH, "set to deep power down\n");
        //PWR_ReleaseLowPowerModeConstraint(PWR_DeepSleep);
        //PWR_SetLowPowerModeConstraint(PWR_DeepPowerDown);
	 
#if (ZIGBEE_USE_FRAMEWORK != 0)
        SecLib_Init();
		if(gRngSuccess_d != RNG_Init())
		{
			DBG_vPrintf(TRUE,"Failed to init RNG\n");
		}
        MEM_Init();
#endif
#if defined(USE_NBU) && (USE_NBU == 1)
        PLATFORM_InitNbu();
        PLATFORM_InitMulticore();
        PLATFORM_FwkSrvInit();
        PLATFORM_SendChipRevision();
        PLATFORM_LoadHwParams(); 
#endif
#endif
        vAppMain();
        DBG_vPrintf(TRUE, "\nmain_task\n");
    }
    while (TRUE)
    {

        ZTIMER_vTask();

        /*
         * suspends CPU operation when the system is idle or puts the device to
         * sleep if there are no activities in progress
         */
        //POWER_SetLowPower( PM_SLEEP_0 , &pmconfig);
#ifndef K32W148EVK
        POWER_EnterSleep();
#else 
        WDOG32_Refresh(wdog32_base);
	 PWR_EnterLowPower(0);
#endif

        if(!gUseRtos_c)
        {
            break;
        }
    }
}
#ifndef K32W1480_SERIES
/****************************************************************************
 *
 * NAME: hardware_init
 *
 * DESCRIPTION:
 *
 *
 * RETURNS:
 * void
 *
 ****************************************************************************/
void hardware_init(void)
{
    static bool_t bColdStarted = FALSE;

    if (FALSE == bColdStarted)
    {
        APP_vSetUpHardware();
        bColdStarted = TRUE;
    }
}
#endif
